
from surveygizmo.api import base


class SurveyStatistic(base.Resource):
    resource_fmt_str = 'survey/%(survey_id)s/surveystatistic/%(statistic_id)s'
    resource_id_keys = ['survey_id', 'statistic_id']

    def list(self, survey_id, **kwargs):
        kwargs.update({'survey_id': survey_id, })
        return super(SurveyStatistic, self).list(**kwargs)

    def get(self, **kwargs):
        raise NotImplementedError()

    def create(self, **kwargs):
        raise NotImplementedError()

    def update(self, **kwargs):
        raise NotImplementedError()

    def copy(self, **kwargs):
        raise NotImplementedError()

    def delete(self, **kwargs):
        raise NotImplementedError()




from __future__ import absolute_import

from .account import Account
from .accountteams import AccountTeams
from .accountuser import AccountUser
from .contact import Contact
from .contactlist import ContactList
from .emailmessage import EmailMessage
from .survey import Survey
from .surveycampaign import SurveyCampaign
from .surveyoption import SurveyOption
from .surveypage import SurveyPage
from .surveyquestion import SurveyQuestion
from .surveyreport import SurveyReport
from .surveyresponse import SurveyResponse
from .surveystatistic import SurveyStatistic


__all__ = [
    'Account',
    'AccountTeams',
    'AccountUser',
    'Contact',
    'ContactList',
    'EmailMessage',
    'Survey',
    'SurveyCampaign',
    'SurveyOption',
    'SurveyPage',
    'SurveyQuestion',
    'SurveyReport',
    'SurveyResponse',
    'SurveyStatistic',
]




from copy import copy


class Resource(object):
    resource_fmt_str = ''
    resource_id_keys = []

    def __init__(self, api):
        self._filters = []
        self.api = api

    def filter(self, field, operator, value):
        """ Add a query filter to be applied to the next API list call for this resource.
            :param field: Field name to filter by
            :type field: str
            :param operator: Operator value
            :type operator: str
            :param value: Value of filter
            :type value: str
            :param object_type: Optional. Checks field for object association
            :type object_type: str

            Known Filters:
                Question                    [question(2)]                   surveyresponse
                Question Option             [question(2), option(10001)]    surveyresponse
                Date Submitted              datesubmitted                   surveyresponse
                Is Test Data                istestdata                      surveyresponse
                Status                      status                          surveyresponse
                Contact ID                  contact_id                      surveyresponse
                Creation Time               createdon                       survey
                Last Modified Time          modifiedon                      survey
                Survey Title                title                           survey
                Type of Project             subtype                         survey
                Team Survey Belongs To      team                            survey
                Status                      status                          survey
                Type of Link                type                            surveycampaign
                Name of Link                name                            surveycampaign
                Secure / Unsecure Link      ssl                             surveycampaign
                Link Created Date           datecreated                     surveycampaign
                Link Last Modified Date     datemodified                    surveycampaign


            Known Operators:
                =           Is equal to (==)
                <>          Is equal to (!=)
                IS NULL     Value is not answered or is blank
                IS NOT NULL Value is answered or is not blank
                in          Value is in comma separated list

            Unknown Operators:
                Not officially mentioned in documentation, but may work.
                ==
                !=
                >=
                <=
                >
                <
        """
        instance = copy(self)

        i = len(instance._filters)
        instance._filters.append({
            'filter[field][%d]' % i: str(field),
            'filter[operator][%d]' % i: str(operator),
            'filter[value][%d]' % i: str(value),
        })
        return instance

    def resultsperpage(self, value):
        """ Set the number of results that will be retrieved by the request. 
            :param value: 'resultsperpage' parameter value for the rest api call
            :type value: str

            Take a look at https://apihelp.surveygizmo.com/help/surveyresponse-sub-object

        """

        instance = copy(self)

        instance._filters.append({
            'resultsperpage': value
        })
        return instance

    def page(self, value):
        """ Set the page which will be returned. 
            :param value: 'page' parameter value for the rest api call
            :type value: str

            Take a look at https://apihelp.surveygizmo.com/help/surveyresponse-sub-object

        """

        instance = copy(self)

        instance._filters.append({
            'page': value
        })
        return instance

    def __copy__(self):
        instance = self.__class__(self.api)
        instance._filters = copy(self._filters)

        return instance

    @property
    def filters(self):
        """
        Returns a merged dictionary of filters.
        """
        params = {}
        for _filter in self._filters:
            params.update(_filter)

        return params

    def list(self, **kwargs):
        kwargs.update(self.filters)
        return self.api.call(*self.prepare(kwargs))

    def get(self, **kwargs):
        return self.api.call(*self.prepare(kwargs))

    def create(self, **kwargs):
        kwargs.update({
            '_method': 'PUT',
        })
        return self.api.call(*self.prepare(kwargs))

    def update(self, **kwargs):
        kwargs.update({
            '_method': 'POST',
        })
        return self.api.call(*self.prepare(kwargs))

    def delete(self, **kwargs):
        kwargs.update({
            '_method': 'DELETE',
        })
        return self.api.call(*self.prepare(kwargs))

    def copy(self, **kwargs):
        kwargs.update({
            '_method': 'POST',
            'copy': 'true',
        })
        return self.api.call(*self.prepare(kwargs))

    def prepare(self, kwargs):
        # format the path part of the url
        ids = {}
        for id_key in self.resource_id_keys:
            ids[id_key] = kwargs.pop(id_key, '')

        path = self.resource_fmt_str % ids

        return path, kwargs




from __future__ import absolute_import

import time
import requests
from .api import base
from .compat import with_metaclass


class ImproperlyConfigured(Exception):
    """
    SurveyGizmo is somehow improperly configured.
    """
    pass


def default_52xhandler(response, resource, url, params):
    """
    Default 52x handler that loops every second until a non 52x response is received.
    :param response: The response of the last executed api request.
    :param resource: The resource of the last executed api request.
    :param url: The url of the last executed api request sans encoded query parameters.
    :param params: The query params of the last executed api request in dictionary format.
    """
    time.sleep(1)
    return resource.execute(url, params)


class Config(object):
    def __init__(self, **kwargs):
        self.api_version = kwargs.get('api_version', 'head')
        self.api_token = kwargs.get('api_token', None)
        self.api_token_secret = kwargs.get('api_token_secret', None)

        self.response_type = kwargs.get('response_type', None)
        self.requests_kwargs = kwargs.get('requests_kwargs', {})
        self.prepare_url = kwargs.get('prepare_url', False)
        self.handler52x = kwargs.get('handler52x', None)

    def validate(self):
        """
        Perform validation check on properties.
        """
        if not self.api_token or not self.api_token_secret:
            raise ImproperlyConfigured("'api_token' and 'api_token_secret' are required for authentication.")

        if self.response_type not in ["json", "pson", "xml", "debug", None]:
            raise ImproperlyConfigured("'%s' is an invalid response_type" % self.response_type)


class APIMeta(type):
    def __init__(cls, name, bases, attrs):
        super(APIMeta, cls).__init__(name, bases, attrs)

        resources = __import__('api', globals(), locals(), [], 1)

        for resource_name in resources.__all__:
            resource = getattr(resources, resource_name)
            if issubclass(resource, base.Resource):

                setattr(cls, resource_name.lower(), cls.prop(resource))

    def prop(self, resource):
        """
        Generate properties for accessing resources.
        """
        @property
        def wrapped(self):
            return resource(self)

        return wrapped


class API(with_metaclass(APIMeta, object)):
    base_url = "https://restapi.surveygizmo.com/"

    def __init__(self, config):
        self.config = config

    def call(self, path, params):
        self.config.validate()
        url = self._prepare_url(path)
        params = self._prepare_params(params)

        if params.pop('_prepare_url', self.config.prepare_url):
            return url, params
        return self.execute(url, params)

    def _prepare_url(self, path):

        # API version and response format
        if self.config.response_type:
            path = "%s.%s" % (path, self.config.response_type)
        path = "%s/%s" % (self.config.api_version, path)

        # full url
        return "%s%s" % (self.base_url, path)

    def _prepare_params(self, params):
        params.update({
            'api_token': self.config.api_token,
            'api_token_secret': self.config.api_token_secret,
        })

        return params

    def execute(self, url, params):
        """ Executes a call to the API.
            :param url: The full url for the api call.
            :param params: Query parameters encoded in the request.
        """
        response = requests.get(url, params=params, **self.config.requests_kwargs)

        if 520 <= response.status_code < 530:
            if self.config.handler52x:
                return self.config.handler52x(response, self, url, params)

        response.raise_for_status()

        if not self.config.response_type:
            return response.json()
        else:
            return response.text


class SurveyGizmo(object):
    """
    SurveyGizmo API client.
    """
    def __init__(self, **kwargs):
        self.config = Config(**kwargs)
        self.api = API(self.config)




from surveygizmo.api import base


class SurveyCampaign(base.Resource):
    resource_fmt_str = 'survey/%(survey_id)s/surveycampaign/%(campaign_id)s'
    resource_id_keys = ['survey_id', 'campaign_id']

    def list(self, survey_id, **kwargs):
        kwargs.update({'survey_id': survey_id, })
        return super(SurveyCampaign, self).list(**kwargs)

    def get(self, survey_id, campaign_id, **kwargs):
        kwargs.update({
            'survey_id': survey_id,
            'campaign_id': campaign_id,
        })
        return super(SurveyCampaign, self).get(**kwargs)

    def create(self, survey_id, name, type, **kwargs):
        kwargs.update({
            'survey_id': survey_id,
            'name': name,
            'type': type,
        })
        return super(SurveyCampaign, self).create(**kwargs)

    def update(self, survey_id, campaign_id, **kwargs):
        kwargs.update({
            'survey_id': survey_id,
            'campaign_id': campaign_id,
        })
        return super(SurveyCampaign, self).update(**kwargs)

    def copy(self, survey_id, campaign_id, **kwargs):
        kwargs.update({
            'survey_id': survey_id,
            'campaign_id': campaign_id,
        })
        return super(SurveyCampaign, self).copy(**kwargs)

    def delete(self, survey_id, campaign_id, **kwargs):
        kwargs.update({
            'survey_id': survey_id,
            'campaign_id': campaign_id,
        })
        return super(SurveyCampaign, self).delete(**kwargs)




from surveygizmo.api import base


class ContactList(base.Resource):
    resource_fmt_str = 'contactlist/%(contactlist_id)s'
    resource_id_keys = ['contactlist_id']

    def list(self, **kwargs):
        return super(ContactList, self).list(**kwargs)

    def get(self, contactlist_id, **kwargs):
        kwargs.update({'contactlist_id': contactlist_id, })
        return super(ContactList, self).get(**kwargs)

    def create(self, listname, **kwargs):
        kwargs.update({
            'listname': listname,
        })
        return super(ContactList, self).create(**kwargs)

    def update(self, contactlist_id, semailaddress, **kwargs):
        kwargs.update({
            'contactlist_id': contactlist_id,
            'semailaddress': semailaddress,
        })
        return super(ContactList, self).update(**kwargs)

    def copy(self, **kwargs):
        raise NotImplementedError()

    def delete(self, **kwargs):
        raise NotImplementedError()




from surveygizmo.api import base


class SurveyQuestion(base.Resource):
    resource_fmt_str = 'survey/%(survey_id)s/surveyquestion/%(question_id)s'
    resource_id_keys = ['survey_id', 'question_id']

    def list(self, survey_id, **kwargs):
        kwargs.update({'survey_id': survey_id, })
        return super(SurveyQuestion, self).list(**kwargs)

    def get(self, survey_id, question_id, **kwargs):
        kwargs.update({
            'survey_id': survey_id,
            'question_id': question_id,
        })
        return super(SurveyQuestion, self).get(**kwargs)

    def create(self, survey_id, page_id, **kwargs):
        # The API deviates a bit with question creation
        path = 'survey/%(survey_id)s/surveypage/%(page_id)s/surveyquestion/' % locals()

        kwargs.update({
            '_method': 'PUT',
        })

        return self.api.call(path, kwargs)

    def update(self, survey_id, question_id, **kwargs):
        kwargs.update({
            'survey_id': survey_id,
            'question_id': question_id,
        })
        return super(SurveyQuestion, self).update(**kwargs)

    def copy(self, **kwargs):
        raise NotImplementedError()

    def delete(self, survey_id, question_id, **kwargs):
        kwargs.update({
            'survey_id': survey_id,
            'question_id': question_id,
        })
        return super(SurveyQuestion, self).delete(**kwargs)




from unittest import TestCase
from surveygizmo import SurveyGizmo

client = SurveyGizmo(api_token='token', api_token_secret='secret', prepare_url=True)
client.api.base_url = ''


class AccountTests(TestCase):
    resource = client.api.account

    def test_list(self):
        with self.assertRaises(NotImplementedError):
            self.resource.list()

    def test_get(self):
        path, params = self.resource.get()
        self.assertEqual(path, 'head/account/')

    def test_create(self):
        with self.assertRaises(NotImplementedError):
            self.resource.create()

    def test_update(self):
        with self.assertRaises(NotImplementedError):
            self.resource.update()

    def test_copy(self):
        with self.assertRaises(NotImplementedError):
            self.resource.copy()

    def test_delete(self):
        with self.assertRaises(NotImplementedError):
            self.resource.delete()


class AccountTeamsTests(TestCase):
    resource = client.api.accountteams

    def test_list(self):
        path, params = self.resource.list()
        self.assertEqual(path, 'head/accountteams/')

    def test_get(self):
        path, params = self.resource.get(1)
        self.assertEqual(path, 'head/accountteams/1')

    def test_create(self):
        path, params = self.resource.create('team')
        self.assertEqual(path, 'head/accountteams/')
        self.assertEqual(params['teamname'], 'team')

    def test_update(self):
        path, params = self.resource.update(1)
        self.assertEqual(path, 'head/accountteams/1')

    def test_copy(self):
        with self.assertRaises(NotImplementedError):
            self.resource.copy()

    def test_delete(self):
        path, params = self.resource.delete(1)
        self.assertEqual(path, 'head/accountteams/1')


class AccountUserTests(TestCase):
    resource = client.api.accountuser

    def test_list(self):
        path, params = self.resource.list()
        self.assertEqual(path, 'head/accountuser/')

    def test_get(self):
        path, params = self.resource.get(1)
        self.assertEqual(path, 'head/accountuser/1')

    def test_create(self):
        path, params = self.resource.create('user@example.com')
        self.assertEqual(path, 'head/accountuser/')
        self.assertEqual(params['email'], 'user@example.com')

    def test_update(self):
        path, params = self.resource.update(1)
        self.assertEqual(path, 'head/accountuser/1')

    def test_copy(self):
        with self.assertRaises(NotImplementedError):
            self.resource.copy()

    def test_delete(self):
        path, params = self.resource.delete(1)
        self.assertEqual(path, 'head/accountuser/1')


class ContactTests(TestCase):
    resource = client.api.contact

    def test_list(self):
        path, params = self.resource.list(1, 1)
        self.assertEqual(path, 'head/survey/1/surveycampaign/1/contact/')

    def test_get(self):
        path, params = self.resource.get(1, 1, 1)
        self.assertEqual(path, 'head/survey/1/surveycampaign/1/contact/1')

    def test_create(self):
        path, params = self.resource.create(1, 1, 'user@example.com')
        self.assertEqual(path, 'head/survey/1/surveycampaign/1/contact/')
        self.assertEqual(params['semailaddress'], 'user@example.com')

    def test_update(self):
        path, params = self.resource.update(1, 1, 1)
        self.assertEqual(path, 'head/survey/1/surveycampaign/1/contact/1')

    def test_copy(self):
        with self.assertRaises(NotImplementedError):
            self.resource.copy()

    def test_delete(self):
        path, params = self.resource.delete(1, 1, 1)
        self.assertEqual(path, 'head/survey/1/surveycampaign/1/contact/1')


class ContactListTests(TestCase):
    resource = client.api.contactlist

    def test_list(self):
        path, params = self.resource.list()
        self.assertEqual(path, 'head/contactlist/')

    def test_get(self):
        path, params = self.resource.get(1)
        self.assertEqual(path, 'head/contactlist/1')

    def test_create(self):
        path, params = self.resource.create('Contact List')
        self.assertEqual(path, 'head/contactlist/')
        self.assertEqual(params['listname'], 'Contact List')

    def test_update(self):
        path, params = self.resource.update(1, 'user@example.com')
        self.assertEqual(path, 'head/contactlist/1')
        self.assertEqual(params['semailaddress'], 'user@example.com')

    def test_copy(self):
        with self.assertRaises(NotImplementedError):
            self.resource.copy()

    def test_delete(self):
        with self.assertRaises(NotImplementedError):
            self.resource.delete()


class EmailMessageTests(TestCase):
    resource = client.api.emailmessage

    def test_list(self):
        path, params = self.resource.list(1, 1)
        self.assertEqual(path, 'head/survey/1/surveycampaign/1/emailmessage/')

    def test_get(self):
        path, params = self.resource.get(1, 1, 1)
        self.assertEqual(path, 'head/survey/1/surveycampaign/1/emailmessage/1')

    def test_create(self):
        path, params = self.resource.create(1, 1)
        self.assertEqual(path, 'head/survey/1/surveycampaign/1/emailmessage/')

    def test_update(self):
        path, params = self.resource.update(1, 1, 1)
        self.assertEqual(path, 'head/survey/1/surveycampaign/1/emailmessage/1')

    def test_copy(self):
        with self.assertRaises(NotImplementedError):
            self.resource.copy()

    def test_delete(self):
        path, params = self.resource.delete(1, 1, 1)
        self.assertEqual(path, 'head/survey/1/surveycampaign/1/emailmessage/1')


class SurveyTests(TestCase):
    resource = client.api.survey

    def test_list(self):
        path, params = self.resource.list()
        self.assertEqual(path, 'head/survey/')

    def test_get(self):
        path, params = self.resource.get(1)
        self.assertEqual(path, 'head/survey/1')

    def test_create(self):
        path, params = self.resource.create('My Survey', 'poll')
        self.assertEqual(path, 'head/survey/')
        self.assertEqual(params['title'], 'My Survey')
        self.assertEqual(params['type'], 'poll')

    def test_update(self):
        path, params = self.resource.update(1)
        self.assertEqual(path, 'head/survey/1')

    def test_copy(self):
        path, params = self.resource.copy(1)
        self.assertEqual(path, 'head/survey/1')

    def test_delete(self):
        path, params = self.resource.delete(1)
        self.assertEqual(path, 'head/survey/1')


class SurveyCampaignTests(TestCase):
    resource = client.api.surveycampaign

    def test_list(self):
        path, params = self.resource.list(1)
        self.assertEqual(path, 'head/survey/1/surveycampaign/')

    def test_get(self):
        path, params = self.resource.get(1, 1)
        self.assertEqual(path, 'head/survey/1/surveycampaign/1')

    def test_create(self):
        path, params = self.resource.create(1, 'My Campaign', 'email')
        self.assertEqual(path, 'head/survey/1/surveycampaign/')
        self.assertEqual(params['name'], 'My Campaign')
        self.assertEqual(params['type'], 'email')

    def test_update(self):
        path, params = self.resource.update(1, 1)
        self.assertEqual(path, 'head/survey/1/surveycampaign/1')

    def test_copy(self):
        path, params = self.resource.copy(1, 1)
        self.assertEqual(path, 'head/survey/1/surveycampaign/1')

    def test_delete(self):
        path, params = self.resource.delete(1, 1)
        self.assertEqual(path, 'head/survey/1/surveycampaign/1')


class SurveyOptionTests(TestCase):
    resource = client.api.surveyoption

    def test_list(self):
        path, params = self.resource.list(1, 1)
        self.assertEqual(path, 'head/survey/1/surveyquestion/1/surveyoption/')

    def test_get(self):
        path, params = self.resource.get(1, 1, 1)
        self.assertEqual(path, 'head/survey/1/surveyquestion/1/surveyoption/1')

    def test_create(self):
        path, params = self.resource.create(1, 1, 'Option', 'Value')
        self.assertEqual(path, 'head/survey/1/surveyquestion/1/surveyoption/')
        self.assertEqual(params['title'], 'Option')
        self.assertEqual(params['value'], 'Value')

    def test_update(self):
        path, params = self.resource.update(1, 1, 1)
        self.assertEqual(path, 'head/survey/1/surveyquestion/1/surveyoption/1')

    def test_copy(self):
        with self.assertRaises(NotImplementedError):
            self.resource.copy()

    def test_delete(self):
        path, params = self.resource.delete(1, 1, 1)
        self.assertEqual(path, 'head/survey/1/surveyquestion/1/surveyoption/1')


class SurveyPageTests(TestCase):
    resource = client.api.surveypage

    def test_list(self):
        path, params = self.resource.list(1)
        self.assertEqual(path, 'head/survey/1/surveypage/')

    def test_get(self):
        path, params = self.resource.get(1, 1)
        self.assertEqual(path, 'head/survey/1/surveypage/1')

    def test_create(self):
        path, params = self.resource.create(1, 'Page 1')
        self.assertEqual(path, 'head/survey/1/surveypage/')
        self.assertEqual(params['title'], 'Page 1')

    def test_update(self):
        path, params = self.resource.update(1, 1)
        self.assertEqual(path, 'head/survey/1/surveypage/1')

    def test_copy(self):
        with self.assertRaises(NotImplementedError):
            self.resource.copy()

    def test_delete(self):
        path, params = self.resource.delete(1, 1)
        self.assertEqual(path, 'head/survey/1/surveypage/1')


class SurveyQuestionTests(TestCase):
    resource = client.api.surveyquestion

    def test_list(self):
        path, params = self.resource.list(1)
        self.assertEqual(path, 'head/survey/1/surveyquestion/')

    def test_get(self):
        path, params = self.resource.get(1, 1)
        self.assertEqual(path, 'head/survey/1/surveyquestion/1')

    def test_create(self):
        path, params = self.resource.create(1, 1)
        self.assertEqual(path, 'head/survey/1/surveypage/1/surveyquestion/')

    def test_update(self):
        path, params = self.resource.update(1, 1)
        self.assertEqual(path, 'head/survey/1/surveyquestion/1')

    def test_copy(self):
        with self.assertRaises(NotImplementedError):
            self.resource.copy()

    def test_delete(self):
        path, params = self.resource.delete(1, 1)
        self.assertEqual(path, 'head/survey/1/surveyquestion/1')


class SurveyReportTests(TestCase):
    resource = client.api.surveyreport

    def test_list(self):
        path, params = self.resource.list(1)
        self.assertEqual(path, 'head/survey/1/surveyreport/')

    def test_get(self):
        path, params = self.resource.get(1, 1)
        self.assertEqual(path, 'head/survey/1/surveyreport/1')

    def test_create(self):
        with self.assertRaises(NotImplementedError):
            self.resource.create()

    def test_update(self):
        path, params = self.resource.update(1, 1)
        self.assertEqual(path, 'head/survey/1/surveyreport/1')
        self.assertEqual(params['copy'], 'false')

    def test_copy(self):
        path, params = self.resource.copy(1, 1)
        self.assertEqual(path, 'head/survey/1/surveyreport/1')
        self.assertEqual(params['copy'], 'true')

    def test_delete(self):
        path, params = self.resource.delete(1, 1)
        self.assertEqual(path, 'head/survey/1/surveyreport/1')


class SurveyResponseTests(TestCase):
    resource = client.api.surveyresponse

    def test_list(self):
        path, params = self.resource.list(1)
        self.assertEqual(path, 'head/survey/1/surveyresponse/')

    def test_get(self):
        path, params = self.resource.get(1, 1)
        self.assertEqual(path, 'head/survey/1/surveyresponse/1')

    def test_create(self):
        path, params = self.resource.create(1)
        self.assertEqual(path, 'head/survey/1/surveyresponse/')

    def test_update(self):
        path, params = self.resource.update(1, 1)
        self.assertEqual(path, 'head/survey/1/surveyresponse/1')

    def test_copy(self):
        with self.assertRaises(NotImplementedError):
            self.resource.copy()

    def test_delete(self):
        path, params = self.resource.delete(1, 1)
        self.assertEqual(path, 'head/survey/1/surveyresponse/1')


class SurveyStatisticTests(TestCase):
    resource = client.api.surveystatistic

    def test_list(self):
        path, params = self.resource.list(1)
        self.assertEqual(path, 'head/survey/1/surveystatistic/')

    def test_get(self):
        with self.assertRaises(NotImplementedError):
            self.resource.get()

    def test_create(self):
        with self.assertRaises(NotImplementedError):
            self.resource.create()

    def test_update(self):
        with self.assertRaises(NotImplementedError):
            self.resource.update()

    def test_copy(self):
        with self.assertRaises(NotImplementedError):
            self.resource.copy()

    def test_delete(self):
        with self.assertRaises(NotImplementedError):
            self.resource.delete()




from surveygizmo.api import base


class SurveyOption(base.Resource):
    resource_fmt_str = 'survey/%(survey_id)s/surveyquestion/%(question_id)s/surveyoption/%(option_id)s'
    resource_id_keys = ['survey_id', 'question_id', 'option_id']

    def list(self, survey_id, question_id, **kwargs):
        kwargs.update({
            'survey_id': survey_id,
            'question_id': question_id,
        })
        return super(SurveyOption, self).list(**kwargs)

    def get(self, survey_id, question_id, option_id, **kwargs):
        kwargs.update({
            'survey_id': survey_id,
            'question_id': question_id,
            'option_id': option_id,
        })
        return super(SurveyOption, self).get(**kwargs)

    def create(self, survey_id, question_id, title, value, **kwargs):
        kwargs.update({
            'survey_id': survey_id,
            'question_id': question_id,
            'title': title,
            'value': value,
        })
        return super(SurveyOption, self).create(**kwargs)

    def update(self, survey_id, question_id, option_id, **kwargs):
        kwargs.update({
            'survey_id': survey_id,
            'question_id': question_id,
            'option_id': option_id,
        })
        return super(SurveyOption, self).update(**kwargs)

    def copy(self, **kwargs):
        raise NotImplementedError()

    def delete(self, survey_id, question_id, option_id, **kwargs):
        kwargs.update({
            'survey_id': survey_id,
            'question_id': question_id,
            'option_id': option_id,
        })
        return super(SurveyOption, self).delete(**kwargs)





# Copied from the 'six'.
def with_metaclass(meta, *bases):
    """Create a base class with a metaclass."""
    # This requires a bit of explanation: the basic idea is to make a dummy
    # metaclass for one level of class instantiation that replaces itself with
    # the actual metaclass.
    class metaclass(meta):

        def __new__(cls, name, this_bases, d):
            return meta(name, bases, d)
    return type.__new__(metaclass, 'temporary_class', (), {})



'''
    SurveyGizmo
    -----------

    A Python Wrapper for SurveyGizmo's restful API service

        >>> from surveygizmo import SurveyGizmo
        >>> sg = SurveyGizmo('v3')
        >>> sg.config.auth_method = 'user:md5'
        >>> sg.config.username = ...
        >>> sg.config.md5_hash = ...

        ...

        >>> surveys = sg.api.survey.list()
        >>> print surveys

'''

from .surveygizmo import SurveyGizmo, ImproperlyConfigured

__all__ = ['SurveyGizmo', 'ImproperlyConfigured']




from surveygizmo.api import base


class SurveyPage(base.Resource):
    resource_fmt_str = 'survey/%(survey_id)s/surveypage/%(page_id)s'
    resource_id_keys = ['survey_id', 'page_id']

    def list(self, survey_id, **kwargs):
        kwargs.update({'survey_id': survey_id, })
        return super(SurveyPage, self).list(**kwargs)

    def get(self, survey_id, page_id, **kwargs):
        kwargs.update({
            'survey_id': survey_id,
            'page_id': page_id,
        })
        return super(SurveyPage, self).get(**kwargs)

    def create(self, survey_id, title, **kwargs):
        kwargs.update({
            'survey_id': survey_id,
            'title': title,
        })
        return super(SurveyPage, self).create(**kwargs)

    def update(self, survey_id, page_id, **kwargs):
        kwargs.update({
            'survey_id': survey_id,
            'page_id': page_id,
        })
        return super(SurveyPage, self).update(**kwargs)

    def copy(self, **kwargs):
        raise NotImplementedError()

    def delete(self, survey_id, page_id, **kwargs):
        kwargs.update({
            'survey_id': survey_id,
            'page_id': page_id,
        })
        return super(SurveyPage, self).delete(**kwargs)




from surveygizmo.api import base


class Account(base.Resource):
    resource_fmt_str = 'account/'
    resource_id_keys = []

    def list(self, **kwargs):
        raise NotImplementedError()

    def get(self, **kwargs):
        return super(Account, self).get(**kwargs)

    def create(self, **kwargs):
        raise NotImplementedError()

    def update(self, **kwargs):
        raise NotImplementedError()

    def copy(self, **kwargs):
        raise NotImplementedError()

    def delete(self, **kwargs):
        raise NotImplementedError()




from unittest import TestCase
from surveygizmo import SurveyGizmo

client = SurveyGizmo(api_token='token', api_token_secret='secret', prepare_url=True)
client.api.base_url = ''


class ResourceTests(TestCase):
    def test_authentication(self):
        _, params = client.api.survey.get(1)
        self.assertEqual(params['api_token'], 'token')
        self.assertEqual(params['api_token_secret'], 'secret')

    def test_methods(self):
        _, params = client.api.survey.list()
        self.assertNotIn('_method', params)

        _, params = client.api.survey.get(1)
        self.assertNotIn('_method', params)

        _, params = client.api.survey.create('My Survey', 'poll')
        self.assertEqual(params['_method'], 'PUT')

        _, params = client.api.survey.update(1)
        self.assertEqual(params['_method'], 'POST')

        _, params = client.api.survey.copy(1)
        self.assertEqual(params['_method'], 'POST')

        _, params = client.api.survey.delete(1)
        self.assertEqual(params['_method'], 'DELETE')


class FilteringTests(TestCase):
    def test_copy_on_filter(self):
        resource1 = client.api.survey
        resource2 = resource1.filter('field', 'op', 'value')

        self.assertIsNot(resource1, resource2)

    def test_filter_chaining(self):
        resource1 = client.api.survey
        resource2 = resource1.filter('field', 'op', 'value')
        resource3 = resource2.filter('field', 'op', 'value')

        self.assertEqual(len(resource1._filters), 0)
        self.assertEqual(len(resource2._filters), 1)
        self.assertEqual(len(resource3._filters), 2)

    def test_list_filters(self):
        _, params = client.api.survey.filter('field', 'op', 'value').list()

        self.assertEqual(params['filter[field][0]'], 'field')
        self.assertEqual(params['filter[operator][0]'], 'op')
        self.assertEqual(params['filter[value][0]'], 'value')

    def test_duplicate_calls(self):
        resource = client.api.survey.filter('field', 'op', 'value')

        _, params = resource.list()

        self.assertEqual(params['filter[field][0]'], 'field')
        self.assertEqual(params['filter[operator][0]'], 'op')
        self.assertEqual(params['filter[value][0]'], 'value')

        _, params = resource.list()

        self.assertEqual(params['filter[field][0]'], 'field')
        self.assertEqual(params['filter[operator][0]'], 'op')
        self.assertEqual(params['filter[value][0]'], 'value')

    def test_no_persistence(self):
        _, params = client.api.survey.filter('field', 'op', 'value').list()

        self.assertEqual(params['filter[field][0]'], 'field')
        self.assertEqual(params['filter[operator][0]'], 'op')
        self.assertEqual(params['filter[value][0]'], 'value')

        _, params = client.api.survey.list()
        self.assertNotIn('filter[field][0]', params)
        self.assertNotIn('filter[operator][0]', params)
        self.assertNotIn('filter[value][0]', params)

    def test_pagination(self):
        _, params = client.api.surveyresponse.resultsperpage('3').list(1)
        self.assertEqual(params['resultsperpage'], '3')

        _, params = client.api.surveyresponse.page('5').list(1)
        self.assertEqual(params['page'], '5')




from surveygizmo.api import base


class AccountUser(base.Resource):
    resource_fmt_str = 'accountuser/%(accountuser_id)s'
    resource_id_keys = ['accountuser_id']

    def list(self, **kwargs):
        return super(AccountUser, self).list(**kwargs)

    def get(self, accountuser_id, **kwargs):
        kwargs.update({'accountuser_id': accountuser_id, })
        return super(AccountUser, self).get(**kwargs)

    def create(self, email, **kwargs):
        kwargs.update({
            'email': email,
        })
        return super(AccountUser, self).create(**kwargs)

    def update(self, accountuser_id, **kwargs):
        kwargs.update({'accountuser_id': accountuser_id, })
        return super(AccountUser, self).update(**kwargs)

    def copy(self, **kwargs):
        raise NotImplementedError()

    def delete(self, accountuser_id, **kwargs):
        kwargs.update({'accountuser_id': accountuser_id, })
        return super(AccountUser, self).delete(**kwargs)




from surveygizmo.api import base


class SurveyReport(base.Resource):
    resource_fmt_str = 'survey/%(survey_id)s/surveyreport/%(report_id)s'
    resource_id_keys = ['survey_id', 'report_id']

    def list(self, survey_id, **kwargs):
        kwargs.update({'survey_id': survey_id, })
        return super(SurveyReport, self).list(**kwargs)

    def get(self, survey_id, report_id, **kwargs):
        kwargs.update({
            'survey_id': survey_id,
            'report_id': report_id,
        })
        return super(SurveyReport, self).get(**kwargs)

    def create(self, **kwargs):
        raise NotImplementedError()

    def update(self, survey_id, report_id, **kwargs):
        kwargs.update({
            'survey_id': survey_id,
            'report_id': report_id,

            # copy is required, even if false?
            'copy': 'false',
        })
        return super(SurveyReport, self).update(**kwargs)

    def copy(self, survey_id, report_id, **kwargs):
        kwargs.update({
            'survey_id': survey_id,
            'report_id': report_id,
        })
        return super(SurveyReport, self).copy(**kwargs)

    def delete(self, survey_id, report_id, **kwargs):
        kwargs.update({
            'survey_id': survey_id,
            'report_id': report_id,
        })
        return super(SurveyReport, self).delete(**kwargs)



import os
import unittest
from setuptools import setup, find_packages

PATH = os.path.join(os.path.dirname(__file__), 'README.md')

try:
    import pypandoc
    README = pypandoc.convert(PATH, 'rst')
except ImportError:
    README = open(PATH).read()

os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))


def test_suite():
    test_loader = unittest.TestLoader()
    test_suite = test_loader.discover('tests', pattern='test_*.py')
    return test_suite


setup(
    name='SurveyGizmo',
    version='1.2.2',
    description='A Python Wrapper for SurveyGizmo\'s restful API service.',
    long_description=README,
    author='Ryan P Kilby',
    author_email='rpkilby@ncsu.edu',
    keywords="Survey Gizmo SurveyGizmo surveygizmo",
    url='https://github.com/ITNG/SurveyGizmo/',
    packages=find_packages(),
    install_requires=['requests'],
    test_suite='setup.test_suite',
    license='BSD License',
    classifiers=(
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ),
)




from unittest import TestCase
from surveygizmo import SurveyGizmo
from surveygizmo import api

client = SurveyGizmo(api_token='token', api_token_secret='secret', prepare_url=True)
client.api.base_url = ''


class APIMetaTests(TestCase):
    def test_resource_properties(self):
        # ensure that we can inspect the API to look at the available resources
        self.assertIn('surveycampaign', dir(client.api))

        # ensure that properties are returning correct types
        self.assertIsInstance(client.api.account, api.account.Account)
        self.assertIsInstance(client.api.accountteams, api.accountteams.AccountTeams)
        self.assertIsInstance(client.api.accountuser, api.accountuser.AccountUser)
        self.assertIsInstance(client.api.contact, api.contact.Contact)
        self.assertIsInstance(client.api.contactlist, api.contactlist.ContactList)
        self.assertIsInstance(client.api.emailmessage, api.emailmessage.EmailMessage)
        self.assertIsInstance(client.api.survey, api.survey.Survey)
        self.assertIsInstance(client.api.surveycampaign, api.surveycampaign.SurveyCampaign)
        self.assertIsInstance(client.api.surveyoption, api.surveyoption.SurveyOption)
        self.assertIsInstance(client.api.surveypage, api.surveypage.SurveyPage)
        self.assertIsInstance(client.api.surveyquestion, api.surveyquestion.SurveyQuestion)
        self.assertIsInstance(client.api.surveyreport, api.surveyreport.SurveyReport)
        self.assertIsInstance(client.api.surveyresponse, api.surveyresponse.SurveyResponse)
        self.assertIsInstance(client.api.surveystatistic, api.surveystatistic.SurveyStatistic)

    def test_new_resource_on_access(self):
        # new invocations should return new Resources. This is necessary
        # so that filtering is not global.
        self.assertIsNot(client.api.survey, client.api.survey)


class APITests(TestCase):
    def test_prepare_url_config(self):
        self.assertTrue(client.config.prepare_url)

        _, params = client.api.survey.list()
        self.assertNotIn('_prepare_url', params)

    def test_prepare_url_override(self):
        client = SurveyGizmo(api_token='token', api_token_secret='secret', prepare_url=False)
        client.api.base_url = ''

        _, params = client.api.survey.list(_prepare_url=True)
        self.assertNotIn('_prepare_url', params)




from surveygizmo.api import base


class AccountTeams(base.Resource):
    resource_fmt_str = 'accountteams/%(accountteams_id)s'
    resource_id_keys = ['accountteams_id']

    def list(self, **kwargs):
        return super(AccountTeams, self).list(**kwargs)

    def get(self, accountteams_id, **kwargs):
        kwargs.update({'accountteams_id': accountteams_id, })
        return super(AccountTeams, self).get(**kwargs)

    def create(self, teamname, **kwargs):
        kwargs.update({
            'teamname': teamname,
        })
        return super(AccountTeams, self).create(**kwargs)

    def update(self, accountteams_id, **kwargs):
        kwargs.update({'accountteams_id': accountteams_id, })
        return super(AccountTeams, self).update(**kwargs)

    def copy(self, **kwargs):
        raise NotImplementedError()

    def delete(self, accountteams_id, **kwargs):
        kwargs.update({'accountteams_id': accountteams_id, })
        return super(AccountTeams, self).delete(**kwargs)
